@@ -3,7 +3,7 @@ Copyright (c) 2016 Corinna Vinschen <corinna@vinschen.de>
 Modified (m) 2017 Thomas Wolff: revise Unicode and locale/wchar handling
  */
 /* Modified (m) 2017 Thomas Wolff: revise Unicode and locale/wchar handling */
-#include <_ansi.h>
+#define _DEFAULT_SOURCE
 #include <ctype.h>
 #include <wctype.h>
 #include "local.h"
@@ -12,6 +12,7 @@ Modified (m) 2017 Thomas Wolff: revise Unicode and locale/wchar handling
 int
 iswpunct_l (wint_t c, struct __locale_t *locale)
 {
+  (void) locale;
 #ifdef _MB_CAPABLE
   //return !iswalnum (c) && iswgraph (c);
   c = _jp2uc_l (c, locale);
