@@ -2,7 +2,7 @@
 Copyright (c) 2016 Corinna Vinschen <corinna@vinschen.de> 
 Modified (m) 2017 Thomas Wolff: revise Unicode and locale/wchar handling
  */
-#include <_ansi.h>
+#define _DEFAULT_SOURCE
 #include <ctype.h>
 
 #undef toascii_l
@@ -10,5 +10,6 @@ Modified (m) 2017 Thomas Wolff: revise Unicode and locale/wchar handling
 int
 toascii_l (int c, struct __locale_t *locale)
 {
+  (void) locale;
   return c & 0177;
 }
