@@ -28,15 +28,11 @@
 #if defined (ICONV_TO_UCS_CES_EUC) \
  || defined (ICONV_FROM_UCS_CES_EUC)
 
-#include <_ansi.h>
-#include <newlib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <sys/types.h>
 #include "../lib/local.h"
-#include "../lib/ucsconv.h"
-#include "../lib/encnames.h"
 #include "../ccs/ccsnames.h"
 
 #define TYPE_EUC_JP 0
