@@ -28,17 +28,15 @@
 #if defined (ICONV_TO_UCS_CES_TABLE) \
  || defined (ICONV_FROM_UCS_CES_TABLE)
  
-#include <_ansi.h>
-#include <newlib.h>
 #include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <stdio.h>
 #include <sys/iconvnls.h>
 #include "../lib/endian.h"
 #include "../lib/local.h"
-#include "../lib/ucsconv.h"
 #include "../ccs/ccs.h"
 
 /*
