@@ -23,8 +23,6 @@
  * SUCH DAMAGE.
  */
 #define _DEFAULT_SOURCE
-#include <_ansi.h>
-#include <newlib.h>
 #include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
