@@ -9,8 +9,8 @@ duplicated in all such forms.
 This file is distributed WITHOUT ANY WARRANTY; without even the implied
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
+#include <sys/features.h>
 #ifndef __IEEE_BIG_ENDIAN
-#include <picolibc.h>
 #ifndef __IEEE_LITTLE_ENDIAN
 
 /* This file can define macros to choose variations of the IEEE float
@@ -146,6 +146,13 @@ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #endif
 #ifdef __ARM_FP
 # define _SUPPORTS_ERREXCEPT
+#else
+#ifdef __clang__
+#include <float.h>
+/* Clang accesses FPSR for FLT_ROUNDS with soft float target */
+#undef FLT_ROUNDS
+#define FLT_ROUNDS 1
+#endif
 #endif
 /* As per ISO/IEC TS 18661 '__FLT_EVAL_METHOD__' will be defined to 16
    (if compiling with +fp16 support) so it can't be used by math.h to
@@ -397,6 +404,10 @@ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #endif
 #endif
 
+#ifdef __ARC64__
+#define __IEEE_LITTLE_ENDIAN
+#endif
+
 #ifdef __CRX__
 #define __IEEE_LITTLE_ENDIAN
 #endif
@@ -589,9 +600,6 @@ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #define __IEEE_BIG_ENDIAN
 #endif
 
-#ifdef __CYGWIN__
-#define __OBSOLETE_MATH_DEFAULT 0
-#endif
 
 #ifndef __OBSOLETE_MATH_DEFAULT
 #define __OBSOLETE_MATH_DEFAULT 1
