@@ -29,26 +29,58 @@ SUCH DAMAGE.
 #ifndef _SYS_UNISTD_H
 #define _SYS_UNISTD_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-#include <_ansi.h>
+#include <sys/cdefs.h>
 #define __need_size_t
 #define __need_ptrdiff_t
-#include <sys/cdefs.h>
-#include <sys/types.h>
-#include <sys/_types.h>
+#define __need_NULL
 #include <stddef.h>
+#include <sys/_types.h>
 
+_BEGIN_STD_C
+
+
+#ifndef _USECONDS_T_DECLARED
+typedef	__useconds_t	useconds_t;	/* microseconds (unsigned) */
+#define	_USECONDS_T_DECLARED
+#endif
+
+#ifndef _INTPTR_T_DECLARED
+typedef __intptr_t intptr_t;
+#define _INTPTR_T_DECLARED
+#endif
+
+#ifndef _SSIZE_T_DECLARED
+typedef _ssize_t ssize_t;
+#define	_SSIZE_T_DECLARED
+#endif
+
+#ifndef _OFF_T_DECLARED
+typedef __off_t off_t;
+#define	_OFF_T_DECLARED
+#endif
+
+#ifndef _UID_T_DECLARED
+typedef	__uid_t		uid_t;		/* user id */
+#define	_UID_T_DECLARED
+#endif
+
+#ifndef _GID_T_DECLARED
+typedef	__gid_t		gid_t;		/* group id */
+#define	_GID_T_DECLARED
+#endif
+
+#ifndef _PID_T_DECLARED
+typedef	__pid_t		pid_t;		/* process id */
+#define	_PID_T_DECLARED
+#endif
 extern char **environ;
 
-void	_exit (int __status) _ATTRIBUTE ((__noreturn__));
+_Noreturn void	_exit (int __status);
 
 int	access (const char *__path, int __amode);
 unsigned  alarm (unsigned __secs);
+int     brk(void*);
 int     chdir (const char *__path);
-int     chmod (const char *__path, mode_t __mode);
 int     chown (const char *__path, uid_t __owner, gid_t __group);
 #if __BSD_VISIBLE || (__XSI_VISIBLE >= 4 && __POSIX_VISIBLE < 200112)
 int     chroot (const char *__path);
@@ -102,9 +134,6 @@ int	faccessat (int __dirfd, const char *__path, int __mode, int __flags);
 #if __BSD_VISIBLE || __XSI_VISIBLE >= 4 || __POSIX_VISIBLE >= 200809
 int     fchdir (int __fildes);
 #endif
-#if __POSIX_VISIBLE >= 199309
-int     fchmod (int __fildes, mode_t __mode);
-#endif
 #if __BSD_VISIBLE || __XSI_VISIBLE >= 4 || __POSIX_VISIBLE >= 200809
 int     fchown (int __fildes, uid_t __owner, gid_t __group);
 #endif
@@ -258,16 +287,10 @@ int     vhangup (void);
 #endif
 ssize_t write (int __fd, const void *__buf, size_t __nbyte);
 
-#ifdef __CYGWIN__
-# define __UNISTD_GETOPT__
-# include <getopt.h>
-# undef __UNISTD_GETOPT__
-#else
 extern char *optarg;			/* getopt(3) external variables */
 extern int optind, opterr, optopt;
 int	 getopt(int, char * const [], const char *);
 extern int optreset;			/* getopt(3) external variable */
-#endif
 
 #if __BSD_VISIBLE || (__XSI_VISIBLE >= 4 && __POSIX_VISIBLE < 200809)
 pid_t   vfork (void);
@@ -282,13 +305,14 @@ pid_t   getpid (void);
 int	isatty (int __fildes);
 int     link (const char *__path1, const char *__path2);
 _off_t  lseek (int __fildes, _off_t __offset, int __whence);
-#ifdef __LARGE64_FILES
-_off64_t lseek64 (int __filedes, _off64_t __offset, int __whence);
-#endif
 void *  sbrk (ptrdiff_t __incr);
 int     unlink (const char *__path);
 #endif
 
+#if __LARGEFILE64_VISIBLE
+_off64_t lseek64 (int __filedes, _off64_t __offset, int __whence);
+#endif
+
 #if !defined(__INSIDE_CYGWIN__)
 #if __POSIX_VISIBLE >= 200112 || __XSI_VISIBLE >= 500
 int     ftruncate (int __fd, off_t __length);
@@ -351,6 +375,12 @@ int	unlinkat (int, const char *, int);
 #define STDOUT_FILENO   1       /* standard output file descriptor */
 #define STDERR_FILENO   2       /* standard error file descriptor */
 
+#ifndef _POSIX2_RE_DUP_MAX
+/* The maximum number of repeated occurrences of a regular expression
+ *    permitted when using the interval notation `\{M,N\}'.  */
+#define _POSIX2_RE_DUP_MAX              255
+#endif /* _POSIX2_RE_DUP_MAX  */
+
 /*
  *  sysconf values per IEEE Std 1003.1, 2008 Edition
  */
@@ -536,13 +566,6 @@ int	unlinkat (int, const char *, int);
 #define _PC_REC_MIN_XFER_SIZE            18
 #define _PC_REC_XFER_ALIGN               19
 #define _PC_TIMESTAMP_RESOLUTION         20
-#ifdef __CYGWIN__
-/* Ask for POSIX permission bits support. */
-#define _PC_POSIX_PERMISSIONS            90
-/* Ask for full POSIX permission support including uid/gid settings. */
-#define _PC_POSIX_SECURITY               91
-#define _PC_CASE_INSENSITIVE             92
-#endif
 
 /*
  *  confstr values per IEEE Std 1003.1, 2004 Edition
@@ -604,9 +627,7 @@ int	unlinkat (int, const char *, int);
 #define _CS_LFS_LINTFLAGS                     24
 #endif
 
-#ifdef __cplusplus
-}
-#endif
+_END_STD_C
 
 #if __SSP_FORTIFY_LEVEL > 0
 #include <ssp/unistd.h>
