@@ -40,32 +40,33 @@ SUCH DAMAGE.
 #ifndef _TIME_H_
 #define _TIME_H_
 
-#include "_ansi.h"
 #include <sys/cdefs.h>
 
 #define __need_size_t
 #define __need_NULL
 #include <stddef.h>
-
-/* Get _CLOCKS_PER_SEC_ */
-#include <machine/time.h>
-
-#ifndef _CLOCKS_PER_SEC_
-#define _CLOCKS_PER_SEC_ 1000
-#endif
-
-#define CLOCKS_PER_SEC _CLOCKS_PER_SEC_
-#define CLK_TCK CLOCKS_PER_SEC
-
-#include <sys/types.h>
-#include <sys/timespec.h>
-
-#if __POSIX_VISIBLE >= 200809
-#include <sys/_locale.h>
-#endif
+#include <sys/_types.h>
+#include <sys/_timespec.h>
 
 _BEGIN_STD_C
 
+#ifndef _CLOCKS_PER_SEC_
+#define _CLOCKS_PER_SEC_ 1000000
+#endif
+
+#ifndef CLOCKS_PER_SEC
+#define CLOCKS_PER_SEC _CLOCKS_PER_SEC_
+#endif
+
+#ifndef TIME_UTC
+#define TIME_UTC 1
+#endif
+
+#ifndef _CLOCK_T_DECLARED
+typedef	_CLOCK_T_	clock_t;
+#define	_CLOCK_T_DECLARED
+#endif
+
 struct tm
 {
   int	tm_sec;
@@ -85,195 +86,46 @@ struct tm
 #endif
 };
 
-clock_t	   clock (void);
-double	   difftime (time_t _time2, time_t _time1);
-time_t	   mktime (struct tm *_timeptr);
-time_t	   time (time_t *_timer);
-#ifndef _REENT_ONLY
-char	  *asctime (const struct tm *_tblock);
-char	  *ctime (const time_t *_time);
-struct tm *gmtime (const time_t *_timer);
-struct tm *localtime (const time_t *_timer);
-#endif
-size_t	   strftime (char *__restrict _s,
-			     size_t _maxsize, const char *__restrict _fmt,
-			     const struct tm *__restrict _t);
-
-#if __POSIX_VISIBLE >= 200809
-extern size_t strftime_l (char *__restrict _s, size_t _maxsize,
-			  const char *__restrict _fmt,
-			  const struct tm *__restrict _t, locale_t _l);
-#endif
-
-char	  *asctime_r 	(const struct tm *__restrict,
-				 char *__restrict);
-char	  *ctime_r 	(const time_t *, char *);
-struct tm *gmtime_r 	(const time_t *__restrict,
-				 struct tm *__restrict);
-struct tm *localtime_r 	(const time_t *__restrict,
-				 struct tm *__restrict);
-
-_END_STD_C
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-#if __XSI_VISIBLE
-char      *strptime (const char *__restrict,
-				 const char *__restrict,
-				 struct tm *__restrict);
-#endif
-#if __GNU_VISIBLE
-char *strptime_l (const char *__restrict, const char *__restrict,
-		  struct tm *__restrict, locale_t);
-#endif
-
 #if __POSIX_VISIBLE
-void      tzset 	(void);
+
+#ifndef CLK_TCK
+#define CLK_TCK CLOCKS_PER_SEC
 #endif
 
-/* getdate functions */
-
-#ifdef HAVE_GETDATE
-#if __XSI_VISIBLE >= 4
-#ifndef _REENT_ONLY
-extern NEWLIB_THREAD_LOCAL int getdate_err;
-
-struct tm *	getdate (const char *);
-/* getdate_err is set to one of the following values to indicate the error.
-     1  the DATEMSK environment variable is null or undefined,
-     2  the template file cannot be opened for reading,
-     3  failed to get file status information,
-     4  the template file is not a regular file,
-     5  an error is encountered while reading the template file,
-     6  memory allication failed (not enough memory available),
-     7  there is no line in the template that matches the input,
-     8  invalid input specification  */
-#endif /* !_REENT_ONLY */
-#endif /* __XSI_VISIBLE >= 4 */
-
-#if __GNU_VISIBLE
-/* getdate_r returns the error code as above */
-int		getdate_r (const char *, struct tm *);
-#endif /* __GNU_VISIBLE */
-#endif /* HAVE_GETDATE */
-
-/* defines for the opengroup specifications Derived from Issue 1 of the SVID.  */
-#if __SVID_VISIBLE || __XSI_VISIBLE
-extern __IMPORT long _timezone;
-extern __IMPORT int _daylight;
-#endif
-#if __POSIX_VISIBLE
-extern __IMPORT char *_tzname[2];
-
-/* POSIX defines the external tzname being defined in time.h */
-#ifndef tzname
-#define tzname _tzname
-#endif
-#endif /* __POSIX_VISIBLE */
-
-#ifdef __cplusplus
-}
-#endif
-
-#include <sys/features.h>
-
-#ifdef __CYGWIN__
-#include <cygwin/time.h>
-#endif /*__CYGWIN__*/
-
-#if defined(_POSIX_TIMERS)
-
-#include <signal.h>
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/* Clocks, P1003.1b-1993, p. 263 */
-
-int clock_settime (clockid_t clock_id, const struct timespec *tp);
-int clock_gettime (clockid_t clock_id, struct timespec *tp);
-int clock_getres (clockid_t clock_id, struct timespec *res);
-
-/* Create a Per-Process Timer, P1003.1b-1993, p. 264 */
-
-int timer_create (clockid_t clock_id,
- 	struct sigevent *__restrict evp,
-	timer_t *__restrict timerid);
-
-/* Delete a Per_process Timer, P1003.1b-1993, p. 266 */
-
-int timer_delete (timer_t timerid);
-
-/* Per-Process Timers, P1003.1b-1993, p. 267 */
-
-int timer_settime (timer_t timerid, int flags,
-	const struct itimerspec *__restrict value,
-	struct itimerspec *__restrict ovalue);
-int timer_gettime (timer_t timerid, struct itimerspec *value);
-int timer_getoverrun (timer_t timerid);
-
-/* High Resolution Sleep, P1003.1b-1993, p. 269 */
-
-int nanosleep (const struct timespec  *rqtp, struct timespec *rmtp);
-
-#ifdef __cplusplus
-}
-#endif
-#endif /* _POSIX_TIMERS */
-
-#if defined(_POSIX_CLOCK_SELECTION)
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-int clock_nanosleep (clockid_t clock_id, int flags,
-	const struct timespec *rqtp, struct timespec *rmtp);
-
-#ifdef __cplusplus
-}
-#endif
-
-#endif /* _POSIX_CLOCK_SELECTION */
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/* CPU-time Clock Attributes, P1003.4b/D8, p. 54 */
-
-/* values for the clock enable attribute */
-
-#define CLOCK_ENABLED  1  /* clock is enabled, i.e. counting execution time */
-#define CLOCK_DISABLED 0  /* clock is disabled */
-
-/* values for the pthread cputime_clock_allowed attribute */
-
-#define CLOCK_ALLOWED    1 /* If a thread is created with this value a */
-                           /*   CPU-time clock attached to that thread */
-                           /*   shall be accessible. */
-#define CLOCK_DISALLOWED 0 /* If a thread is created with this value, the */
-                           /*   thread shall not have a CPU-time clock */
-                           /*   accessible. */
+#define CLOCK_REALTIME		((clockid_t) 1)
 
 /* Flag indicating time is "absolute" with respect to the clock
    associated with a time.  Value 4 is historic. */
 
 #define TIMER_ABSTIME	4
 
-/* Manifest Constants, P1003.1b-1993, p. 262 */
+#include <sys/_locale.h>
+
+#ifndef _CLOCKID_T_DECLARED
+typedef	__clockid_t	clockid_t;
+#define	_CLOCKID_T_DECLARED
+#endif
+
+#ifndef _TIMER_T_DECLARED
+typedef	__timer_t	timer_t;
+#define	_TIMER_T_DECLARED
+#endif
+
+/*
+ * Structure defined by POSIX.1b to be like a itimerval, but with
+ * timespecs. Used in the timer_*() system calls.
+ */
+struct itimerspec {
+	struct timespec  it_interval;
+	struct timespec  it_value;
+};
+
+#endif
 
 #if __GNU_VISIBLE
 #define CLOCK_REALTIME_COARSE	((clockid_t) 0)
 #endif
 
-#define CLOCK_REALTIME		((clockid_t) 1)
-
-/* Manifest Constants, P1003.4b/D8, p. 55 */
-
 #if defined(_POSIX_CPUTIME)
 
 /* When used in a clock or timer function call, this is interpreted as
@@ -318,26 +170,141 @@ extern "C" {
 
 #endif
 
-#if defined(_POSIX_CPUTIME)
-
-/* Accessing a Process CPU-time CLock, P1003.4b/D8, p. 55 */
-
-int clock_getcpuclockid (pid_t pid, clockid_t *clock_id);
-
-#endif /* _POSIX_CPUTIME */
-
-#if defined(_POSIX_CPUTIME) || defined(_POSIX_THREAD_CPUTIME)
-
-/* CPU-time Clock Attribute Access, P1003.4b/D8, p. 56 */
-
-int clock_setenable_attr (clockid_t clock_id, int attr);
-int clock_getenable_attr (clockid_t clock_id, int *attr);
-
-#endif /* _POSIX_CPUTIME or _POSIX_THREAD_CPUTIME */
-
-#ifdef __cplusplus
-}
+/* defines for the opengroup specifications Derived from Issue 1 of the SVID.  */
+#if __SVID_VISIBLE || __XSI_VISIBLE
+extern long _timezone;
+extern int _daylight;
 #endif
 
-#endif /* _TIME_H_ */
+#if __POSIX_VISIBLE
+extern char *tzname[2];
+#endif /* __POSIX_VISIBLE */
 
+char	  *asctime (const struct tm *_tblock);
+
+#if __POSIX_VISIBLE
+#define __ASCTIME_SIZE 26
+
+char	  *asctime_r 	(const struct tm *__restrict,
+                         char [__restrict_arr __min_size(__ASCTIME_SIZE)]);
+#endif
+
+clock_t	   clock (void);
+
+#if defined(_POSIX_CPUTIME)
+int        clock_getcpuclockid (pid_t pid, clockid_t *clock_id);
+#endif /* _POSIX_CPUTIME */
+
+#if __POSIX_VISIBLE
+int        clock_getres (clockid_t clock_id, struct timespec *res);
+
+int        clock_gettime (clockid_t clock_id, struct timespec *tp);
+
+int        clock_nanosleep (clockid_t clock_id, int flags,
+                            const struct timespec *rqtp,
+                            struct timespec *rmtp);
+
+int        clock_settime (clockid_t clock_id, const struct timespec *tp);
+#endif
+
+char	  *ctime (const time_t *_time);
+
+#if __POSIX_VISIBLE
+char	  *ctime_r 	(const time_t *,
+                         char [__restrict_arr __min_size(__ASCTIME_SIZE)]);
+#endif
+
+double	   difftime (time_t _time2, time_t _time1);
+
+#if __XSI_VISIBLE >= 4
+struct tm *getdate (const char *);
+#endif /* __XSI_VISIBLE >= 4 */
+
+#if __GNU_VISIBLE
+extern int getdate_err;
+
+int	   getdate_r (const char *, struct tm *);
+#endif /* __GNU_VISIBLE */
+
+struct tm *gmtime (const time_t *_timer);
+
+#if __POSIX_VISIBLE
+struct tm *gmtime_r (const time_t *__restrict,
+                     struct tm *__restrict);
+#endif
+
+struct tm *localtime (const time_t *_timer);
+
+#if __POSIX_VISIBLE
+struct tm *localtime_r 	(const time_t *__restrict,
+				 struct tm *__restrict);
+#endif
+
+time_t	   mktime (struct tm *_timeptr);
+
+int        nanosleep (const struct timespec  *rqtp, struct timespec *rmtp);
+
+size_t	   strftime (char *__restrict _s,
+                     size_t _maxsize, const char *__restrict _fmt,
+                     const struct tm *__restrict _t);
+
+int        timespec_get(struct timespec *_ts, int _base);
+
+#if __POSIX_VISIBLE
+size_t strftime_l (char *__restrict _s, size_t _maxsize,
+			  const char *__restrict _fmt,
+			  const struct tm *__restrict _t, locale_t _l);
+#endif
+
+#if __XSI_VISIBLE
+char      *strptime (const char *__restrict,
+				 const char *__restrict,
+				 struct tm *__restrict);
+#endif
+#if __GNU_VISIBLE
+char      *strptime_l (const char *__restrict, const char *__restrict,
+                       struct tm *__restrict, locale_t);
+#endif
+
+time_t	   time (time_t *_timer);
+
+#if __BSD_VISIBLE || __SVID_VISIBLE || __GNU_VISIBLE
+time_t	   timegm (struct tm *_timeptr);
+#endif
+
+#if __POSIX_VISIBLE
+struct sigevent;
+int        timer_create (clockid_t clock_id,
+                         struct sigevent *__restrict evp,
+                         timer_t *__restrict timerid);
+
+int        timer_delete (timer_t timerid);
+
+int        timer_getoverrun (timer_t timerid);
+
+int        timer_gettime (timer_t timerid, struct itimerspec *value);
+
+int        timer_settime (timer_t timerid, int flags,
+                          const struct itimerspec *__restrict value,
+                          struct itimerspec *__restrict ovalue);
+#endif
+
+void       tzset (void);
+
+#if __STDC_WANT_LIB_EXT1__ == 1
+#include <sys/_types.h>
+
+#ifndef _ERRNO_T_DEFINED
+typedef __errno_t errno_t;
+#define _ERRNO_T_DEFINED
+#endif
+
+#ifndef _RSIZE_T_DEFINED
+typedef __rsize_t rsize_t;
+#define _RSIZE_T_DEFINED
+#endif
+#endif
+
+_END_STD_C
+
+#endif /* _TIME_H_ */
