@@ -67,8 +67,7 @@ PORTABILITY
 <<duplocale>> is POSIX-1.2008.
 */
 
-#define _DEFAULT_SOURCE
-#include <newlib.h>
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include "setlocale.h"
 
@@ -78,6 +77,10 @@ duplocale (struct __locale_t *locobj)
   struct __locale_t tmp_locale, *new_locale;
   int i;
 
+  (void) locobj;
+  (void) i;
+  (void) new_locale;
+  (void) tmp_locale;
 #ifndef _MB_CAPABLE
   return __get_C_locale ();
 #else /* _MB_CAPABLE */
