@@ -61,8 +61,7 @@ PORTABILITY
 <<freelocale>> is POSIX-1.2008.
 */
 
-#define _DEFAULT_SOURCE
-#include <newlib.h>
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include "setlocale.h"
 
