@@ -17,7 +17,6 @@
 /* No user fns here.  Pesch 15apr92. */
 
 #define _DEFAULT_SOURCE
-#include <_ansi.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -264,41 +263,3 @@ __sinit (void)
   global_stdio_init ();
   __sfp_lock_release ();
 }
-
-#ifndef __SINGLE_THREAD__
-
-
-/* Walkable file locking routine.  */
-static int
-__fp_lock (FILE * fp)
-{
-  if (!(fp->_flags2 & __SNLK))
-    _flockfile (fp);
-
-  return 0;
-}
-
-/* Walkable file unlocking routine.  */
-static int
-__fp_unlock (FILE * fp)
-{
-  if (!(fp->_flags2 & __SNLK))
-    _funlockfile (fp);
-
-  return 0;
-}
-
-void
-__fp_lock_all (void)
-{
-  __sfp_lock_acquire ();
-  (void) _fwalk_sglue (__fp_lock, &__sglue);
-}
-
-void
-__fp_unlock_all (void)
-{
-  (void) _fwalk_sglue (__fp_unlock, &__sglue);
-  __sfp_lock_release ();
-}
-#endif
