@@ -75,8 +75,6 @@ C99, POSIX-1.2008
 */
 
 #define _DEFAULT_SOURCE
-#include <_ansi.h>
-#include <newlib.h>
 #include <ctype.h>
 #include <wctype.h>
 #include <stdio.h>
@@ -91,6 +89,7 @@ C99, POSIX-1.2008
 
 #ifdef INTEGER_ONLY
 #define VFWSCANF vfiwscanf
+__typeof(vfwscanf) vfiwscanf;
 #ifdef STRING_ONLY
 #  define _SVFWSCANF _ssvfiwscanf
 #else
