@@ -2,12 +2,19 @@
   * This program has been placed in the public domain.
   */
 
+#ifdef __GNUC__
+#pragma GCC diagnostic ignored "-Wpragmas"
+#pragma GCC diagnostic ignored "-Wunknown-warning-option"
+#pragma GCC diagnostic ignored "-Wanalyzer-malloc-leak"
+#pragma GCC diagnostic ignored "-Wanalyzer-use-of-uninitialized-value"
+#pragma GCC diagnostic ignored "-Wanalyzer-out-of-bounds"
+#pragma GCC diagnostic ignored "-Warray-bounds"
+#pragma GCC diagnostic ignored "-Wanalyzer-null-dereference"
+#endif
+
 #define _DEFAULT_SOURCE
-#include <newlib.h>
-#include <sys/config.h>
 
 #ifndef _USE_GDTOA
-#include <_ansi.h>
 #include <string.h>
 #include <stdlib.h>
 #include "mprec.h"
@@ -1019,7 +1026,7 @@ m16m (short unsigned int a, short unsigned int *b, short unsigned int *c)
   register unsigned short *pp;
   register unsigned long carry;
   unsigned short *ps;
-  unsigned short p[NI];
+  unsigned short p[NI]= {0};
   unsigned long aa, m;
   int i;
 
@@ -1151,7 +1158,7 @@ static int
 emulm (short unsigned int *a, short unsigned int *b, LDPARMS * ldp)
 {
   unsigned short *p, *q;
-  unsigned short pprod[NI];
+  unsigned short pprod[NI] = {0};
   unsigned short j;
   int i;
   unsigned short *equot = ldp->equot;
@@ -1464,7 +1471,7 @@ static void
 eadd1 (const short unsigned int *a, const short unsigned int *b,
        short unsigned int *c, int subflg, LDPARMS * ldp)
 {
-  unsigned short ai[NI], bi[NI], ci[NI];
+  unsigned short ai[NI] = {0}, bi[NI] = {0}, ci[NI] = {0};
   int i, lost, j, k;
   long lt, lta, ltb;
 
@@ -1575,7 +1582,7 @@ static void
 ediv (const short unsigned int *a, const short unsigned int *b,
       short unsigned int *c, LDPARMS * ldp)
 {
-  unsigned short ai[NI], bi[NI];
+  unsigned short ai[NI] = {0}, bi[NI] = {0};
   int i;
   long lt, lta, ltb;
 
@@ -1681,7 +1688,7 @@ static void
 emul (const short unsigned int *a, const short unsigned int *b,
       short unsigned int *c, LDPARMS * ldp)
 {
-  unsigned short ai[NI], bi[NI];
+  unsigned short ai[NI] = {0}, bi[NI] = {0};
   int i, j;
   long lt, lta, ltb;
 
@@ -1773,7 +1780,7 @@ e113toe (short unsigned int *pe, short unsigned int *y, LDPARMS * ldp)
 {
   register unsigned short r;
   unsigned short *e, *p;
-  unsigned short yy[NI];
+  unsigned short yy[NI] = {0};
   int i;
 
   e = pe;
@@ -1898,7 +1905,7 @@ toe113 (short unsigned int *a, short unsigned int *b)
 static void
 e64toe (short unsigned int *pe, short unsigned int *y, LDPARMS * ldp)
 {
-  unsigned short yy[NI];
+  unsigned short yy[NI] = {0};
   unsigned short *p, *q, *e;
   int i;
 
@@ -2065,7 +2072,7 @@ e53toe (short unsigned int *pe, short unsigned int *y, LDPARMS * ldp)
 
   register unsigned short r;
   register unsigned short *p, *e;
-  unsigned short yy[NI];
+  unsigned short yy[NI] = {0};
   int denorm, k;
 
   e = pe;
@@ -2263,7 +2270,7 @@ e24toe (short unsigned int *pe, short unsigned int *y, LDPARMS * ldp)
 {
   register unsigned short r;
   register unsigned short *p, *e;
-  unsigned short yy[NI];
+  unsigned short yy[NI] = {0};
   int denorm, k;
 
   e = pe;
@@ -2430,7 +2437,7 @@ toe24 (short unsigned int *x, short unsigned int *y)
 static int
 ecmp (const short unsigned int *a, const short unsigned int *b)
 {
-  unsigned short ai[NI], bi[NI];
+  unsigned short ai[NI] = {0}, bi[NI] = {0};
   register unsigned short *p, *q;
   register int i;
   int msign;
@@ -2745,7 +2752,7 @@ _IO_ldtostr (x, string, ndigs, flags, fmt)
      int flags;
      char fmt;
 {
-  unsigned short w[NI];
+  unsigned short w[NI] = {0};
   char *t, *u;
   LDPARMS rnd;
   LDPARMS *ldp = &rnd;
@@ -2793,7 +2800,7 @@ char *
 __ldtoa (long double d, int mode, int ndigits,
        int *decpt, int *sign, char **rve)
 {
-  unsigned short e[NI];
+  unsigned short e[NI] = {0};
   char *s, *p;
   int i, k;
   int orig_ndigits;
@@ -2961,7 +2968,7 @@ stripspaces:
 int
 _ldcheck (long double *d)
 {
-  unsigned short e[NI];
+  unsigned short e[NI] = {0};
   LDPARMS rnd;
   LDPARMS *ldp = &rnd;
 
@@ -2997,7 +3004,7 @@ etoasc (short unsigned int *x, char *string, int ndec, int ndigits,
 	int outformat, LDPARMS * ldp)
 {
   long digit;
-  unsigned short y[NI], t[NI], u[NI], w[NI];
+  unsigned short y[NI] = {0}, t[NI] = {0}, u[NI] = {0}, w[NI] = {0};
   const unsigned short *p, *r, *ten;
   unsigned short sign;
   int i, j, k, expon, rndsav, ndigs;
@@ -3348,7 +3355,7 @@ strtold (char *s, char **se)
 static int
 asctoeg (char *ss, short unsigned int *y, int oprec, LDPARMS * ldp)
 {
-  unsigned short yy[NI], xt[NI], tt[NI];
+  unsigned short yy[NI] = {0}, xt[NI] = {0}, tt[NI] = {0};
   int esign, decflg, sgnflg, nexp, exp, prec, lost;
   int k, trail, c, rndsav;
   long lexp;
