@@ -26,12 +26,6 @@ wctomb_f __eucjp_wctomb;
 wctomb_f __jis_wctomb;
 wctomb_p __iso_wctomb (int val);
 wctomb_p __cp_wctomb (int val);
-#ifdef __CYGWIN__
-wctomb_f __gbk_wctomb;
-wctomb_f __gb18030_wctomb;
-wctomb_f __kr_wctomb;
-wctomb_f __big5_wctomb;
-#endif
 #endif
 
 #define __WCTOMB (__get_current_locale()->wctomb)
@@ -48,12 +42,6 @@ mbtowc_f __eucjp_mbtowc;
 mbtowc_f __jis_mbtowc;
 mbtowc_p __iso_mbtowc (int val);
 mbtowc_p __cp_mbtowc (int val);
-#ifdef __CYGWIN__
-mbtowc_f __gbk_mbtowc;
-mbtowc_f __gb18030_mbtowc;
-mbtowc_f __kr_mbtowc;
-mbtowc_f __big5_mbtowc;
-#endif
 #endif
 
 #define __MBTOWC (__get_current_locale()->mbtowc)
