@@ -80,8 +80,17 @@
  *	down depends on the machine and the number being converted.
  */
 
+#ifdef __GNUC__
+#pragma GCC diagnostic ignored "-Wpragmas"
+#pragma GCC diagnostic ignored "-Wunknown-warning-option"
+#pragma GCC diagnostic ignored "-Wanalyzer-malloc-leak"
+#pragma GCC diagnostic ignored "-Wanalyzer-use-of-uninitialized-value"
+#pragma GCC diagnostic ignored "-Wanalyzer-out-of-bounds"
+#pragma GCC diagnostic ignored "-Warray-bounds"
+#pragma GCC diagnostic ignored "-Wanalyzer-null-dereference"
+#endif
+
 #define _DEFAULT_SOURCE
-#include <_ansi.h>
 #include <stdlib.h>
 #include <string.h>
 #include "mprec.h"
