@@ -30,7 +30,6 @@ THIS SOFTWARE.
  * with " at " changed at "@" and " dot " changed to ".").	*/
 
 #define _DEFAULT_SOURCE
-#include <_ansi.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
@@ -166,7 +165,7 @@ trailz (_Bigint *b)
 	return n;
 	}
 
-_Bigint *
+static _Bigint *
 increment (_Bigint *b)
 {
 	__ULong *x, *xe;
@@ -211,7 +210,7 @@ increment (_Bigint *b)
 	return b;
 	}
 
-int
+static int
 decrement (_Bigint *b)
 {
 	__ULong *x, *xe;
@@ -255,7 +254,7 @@ all_on (_Bigint *b, int n)
 	return 1;
 	}
 
-_Bigint *
+static _Bigint *
 set_ones (_Bigint *b, int n)
 {
 	int k;
