@@ -35,8 +35,6 @@
 
 #include "stdio_private.h"
 
-/* Buffered I/O routines for tiny stdio */
-
 int
 __bufio_flush_locked(FILE *f)
 {
@@ -49,7 +47,7 @@ __bufio_flush_locked(FILE *f)
 		/* Flush everything, drop contents if that doesn't work */
                 buf = bf->buf;
 		while (bf->len) {
-                        ssize_t this = (bf->write) (bf->fd, buf, bf->len);
+                        ssize_t this = bufio_write(bf, buf, bf->len);
 			if (this <= 0) {
                                 bf->len = 0;
                                 return _FDEV_ERR;
@@ -64,8 +62,7 @@ __bufio_flush_locked(FILE *f)
                 backup = bf->len - bf->off;
                 if (backup) {
                         bf->pos -= backup;
-                        if (bf->lseek)
-                                (void) (bf->lseek)(bf->fd, bf->pos, SEEK_SET);
+                        (void) bufio_lseek(bf, bf->pos, SEEK_SET);
                 }
                 bf->len = 0;
                 bf->off = 0;
@@ -84,7 +81,7 @@ int __bufio_fill_locked(FILE *f)
 
         /* Reset read pointer, read some data */
         bf->off = 0;
-        len = (bf->read)(bf->fd, bf->buf, bf->size);
+        len = bufio_read (bf, bf->buf, bf->size);
 
         if (len <= 0) {
                 bf->len = 0;
@@ -168,12 +165,21 @@ again:
 
 	if (bf->off >= bf->len) {
 
-		/* Flush stdout if reading from stdin */
-		if (f == stdin && !flushed && stdout != NULL) {
+		/*
+                 * Flush stdout if reading from stdin.
+                 *
+                 * The odd-looking NULL address checks along with the
+                 * weak attributes for stdin and stdout above avoids
+                 * pulling in stdin/stdout definitions just for this
+                 * check.
+                 */
+                if (!flushed) {
                         flushed = true;
-			__bufio_unlock(f);
-			fflush(stdout);
-                        goto again;
+                        if (&stdin != NULL && &stdout != NULL && f == stdin) {
+                                __bufio_unlock(f);
+                                fflush(stdout);
+                                goto again;
+                        }
 		}
 
                 ret = __bufio_fill_locked(f);
@@ -197,9 +203,6 @@ __bufio_seek(FILE *f, off_t offset, int whence)
 	struct __file_bufio *bf = (struct __file_bufio *) f;
 	off_t ret;
 
-        if (!bf->lseek)
-            return _FDEV_ERR;
-
 	__bufio_lock(f);
         if (__bufio_setdir_locked(f, __SRD) < 0) {
                 ret = _FDEV_ERR;
@@ -222,7 +225,7 @@ __bufio_seek(FILE *f, off_t offset, int whence)
                         }
                         __PICOLIBC_FALLTHROUGH;
                 default:
-                        ret = (bf->lseek)(bf->fd, offset, whence);
+                        ret = bufio_lseek(bf, offset, whence);
                         if (ret >= 0)
                                 bf->pos = ret;
                         /* Flush any buffered data after a real seek */
@@ -297,10 +300,16 @@ __bufio_close(FILE *f)
                 free(bf->buf);
 
 	__bufio_lock_close(f);
-	/* Don't close stdin/stdout/stderr fds */
-	if (bf->fd > 2)
-		(bf->close)(bf->fd);
-	free(f);
+
+        /*
+         * Don't close the fd or free the FILE for things not
+         * generated by fopen or fdopen. These will usually be static
+         * FILE structs defined for stdin/stdout/stderr.
+         */
+        if (bf->bflags & __BFALL) {
+                ret = bufio_close(bf);
+                free(f);
+        }
 	return ret;
 }
 
