@@ -42,10 +42,17 @@
 
 #define __BALL  0x0001          /* bufio buf is allocated by stdio */
 #define __BLBF  0x0002          /* bufio is line buffered */
+#define __BFALL 0x0004          /* FILE is allocated by stdio */
+#define __BFPTR 0x0008          /* funcs need pointers instead of ints */
+
+union __file_bufio_cookie {
+        int	fd;
+        void    *ptr;
+};
 
 struct __file_bufio {
         struct __file_ext xfile;
-	int	fd;
+        const void *ptr;
         uint8_t dir;
         uint8_t bflags;
         __off_t pos;    /* FD position */
@@ -53,10 +60,22 @@ struct __file_bufio {
         int     size;   /* sizeof buf */
 	int	len;    /* valid data in buf */
 	int	off;    /* offset of data in buf */
-        ssize_t (*read)(int fd, void *buf, size_t count);
-        ssize_t (*write)(int fd, const void *buf, size_t count);
-        __off_t (*lseek)(int fd, __off_t offset, int whence);
-        int     (*close)(int fd);
+        union {
+                ssize_t (*read_int)(int fd, void *buf, size_t count);
+                ssize_t (*read_ptr)(void *ptr, void *buf, size_t count);
+        };
+        union {
+                ssize_t (*write_int)(int fd, const void *buf, size_t count);
+                ssize_t (*write_ptr)(void *ptr, const void *buf, size_t count);
+        };
+        union {
+                __off_t (*lseek_int)(int fd, __off_t offset, int whence);
+                __off_t (*lseek_ptr)(void *ptr, __off_t offset, int whence);
+        };
+        union {
+                int     (*close_int)(int fd);
+                int     (*close_ptr)(void *ptr);
+        };
 #ifndef __SINGLE_THREAD__
 	_LOCK_T lock;
 #endif
@@ -68,7 +87,7 @@ struct __file_bufio {
                                         __bufio_flush, __bufio_close,   \
                                         __bufio_seek, __bufio_setvbuf,  \
                                         (_rwflag) | __SBUF),            \
-                .fd = _fd,                                              \
+                .ptr = (void *) (intptr_t) (_fd),                       \
                 .dir = 0,                                               \
                 .bflags = (_bflags),                                    \
                 .pos = 0,                                               \
@@ -76,10 +95,30 @@ struct __file_bufio {
                 .size = _size,                                          \
                 .len = 0,                                               \
                 .off = 0,                                               \
-                .read = _read,                                          \
-                .write = _write,                                        \
-                .lseek = _lseek,                                        \
-                .close = _close,                                        \
+                .read_int = _read,                                      \
+                .write_int = _write,                                    \
+                .lseek_int = _lseek,                                    \
+                .close_int = _close,                                    \
+        }
+
+#define FDEV_SETUP_BUFIO_PTR(_ptr, _buf, _size, _read, _write, _lseek, _close, _rwflag, _bflags) \
+        {                                                               \
+                .xfile = FDEV_SETUP_EXT(__bufio_put, __bufio_get,       \
+                                        __bufio_flush, __bufio_close,   \
+                                        __bufio_seek, __bufio_setvbuf,  \
+                                        (_rwflag) | __SBUF),            \
+                .ptr = _ptr,                                            \
+                .dir = 0,                                               \
+                .bflags = (_bflags) | __BFPTR,                          \
+                .pos = 0,                                               \
+                .buf = _buf,                                            \
+                .size = _size,                                          \
+                .len = 0,                                               \
+                .off = 0,                                               \
+                .read_ptr = _read,                                      \
+                .write_ptr = _write,                                    \
+                .lseek_ptr = _lseek,                                    \
+                .close_ptr = _close,                                    \
         }
 
 static inline void __bufio_lock_init(FILE *f) {
