@@ -39,17 +39,15 @@
 #ifndef _STDIO_H_
 #define	_STDIO_H_ 1
 
-#include "_ansi.h"
-
+#include <sys/cdefs.h>
 #define __need_NULL
 #define __need_size_t
-#define __need_ssize_t
-#include <sys/cdefs.h>
 #include <stddef.h>
-
-#include <inttypes.h>
+#define __need___va_list
 #include <stdarg.h>
-#include <sys/types.h>
+#include <sys/_types.h>
+
+_BEGIN_STD_C
 
 /*
  * This is an internal structure of the library that is subject to be
@@ -74,16 +72,16 @@
 #endif
 
 #if __PICOLIBC_UNGETC_SIZE == 4
-typedef uint32_t __ungetc_t;
+typedef __uint32_t __ungetc_t;
 #endif
 
 #if __PICOLIBC_UNGETC_SIZE == 2
-typedef uint16_t __ungetc_t;
+typedef __uint16_t __ungetc_t;
 #endif
 
 struct __file {
 	__ungetc_t unget;	/* ungetc() buffer */
-	uint8_t	flags;		/* flags, see below */
+	__uint8_t  flags;	/* flags, see below */
 #define __SRD	0x0001		/* OK to read */
 #define __SWR	0x0002		/* OK to write */
 #define __SERR	0x0004		/* found error */
@@ -130,10 +128,14 @@ struct __file_ext {
    \c FILE is the opaque structure that is passed around between the
    various standard IO functions.
 */
+#ifndef ___FILE_DECLARED
 typedef struct __file __FILE;
-#if !defined(__FILE_defined)
+# define __FILE_DECLARED
+#endif
+
+#ifndef _FILE_DECLARED
 typedef __FILE FILE;
-# define __FILE_defined
+#define _FILE_DECLARED
 #endif
 
 /**
@@ -193,10 +195,6 @@ extern FILE *const stderr;
                 .flush = (fl),                  \
 	}
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 FILE *fdevopen(int (*__put)(char, FILE*), int (*__get)(FILE*), int(*__flush)(FILE *));
 int	fclose(FILE *__stream);
 int	fflush(FILE *stream);
@@ -225,14 +223,14 @@ int	putchar(int __c);
 
 int	printf(const char *__fmt, ...) __PRINTF_ATTRIBUTE__(1, 2);
 int	fprintf(FILE *__stream, const char *__fmt, ...) __PRINTF_ATTRIBUTE__(2, 3);
-int	vprintf(const char *__fmt, va_list __ap) __PRINTF_ATTRIBUTE__(1, 0);
-int	vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __PRINTF_ATTRIBUTE__(2, 0);
+int	vprintf(const char *__fmt, __gnuc_va_list __ap) __PRINTF_ATTRIBUTE__(1, 0);
+int	vfprintf(FILE *__stream, const char *__fmt, __gnuc_va_list __ap) __PRINTF_ATTRIBUTE__(2, 0);
 int	sprintf(char *__s, const char *__fmt, ...) __PRINTF_ATTRIBUTE__(2, 3);
 int	snprintf(char *__s, size_t __n, const char *__fmt, ...) __PRINTF_ATTRIBUTE__(3, 4);
-int	vsprintf(char *__s, const char *__fmt, va_list ap) __PRINTF_ATTRIBUTE__(2, 0);
-int	vsnprintf(char *__s, size_t __n, const char *__fmt, va_list ap) __PRINTF_ATTRIBUTE__(3, 0);
+int	vsprintf(char *__s, const char *__fmt, __gnuc_va_list ap) __PRINTF_ATTRIBUTE__(2, 0);
+int	vsnprintf(char *__s, size_t __n, const char *__fmt, __gnuc_va_list ap) __PRINTF_ATTRIBUTE__(3, 0);
 int     asprintf(char **strp, const char *fmt, ...) __PRINTF_ATTRIBUTE__(2,3);
-int     vasprintf(char **strp, const char *fmt, va_list ap) __PRINTF_ATTRIBUTE__(2,0);
+int     vasprintf(char **strp, const char *fmt, __gnuc_va_list ap) __PRINTF_ATTRIBUTE__(2,0);
 
 int	fputs(const char *__str, FILE *__stream);
 int	puts(const char *__str);
@@ -248,10 +246,10 @@ int	ungetc(int __c, FILE *__stream);
 
 int	scanf(const char *__fmt, ...) __FORMAT_ATTRIBUTE__(scanf, 1, 2);
 int	fscanf(FILE *__stream, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(scanf, 2, 3);
-int	vscanf(const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 1, 0);
-int	vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	vscanf(const char *__fmt, __gnuc_va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 1, 0);
+int	vfscanf(FILE *__stream, const char *__fmt, __gnuc_va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
 int	sscanf(const char *__buf, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(scanf, 2, 3);
-int	vsscanf(const char *__buf, const char *__fmt, va_list ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	vsscanf(const char *__buf, const char *__fmt, __gnuc_va_list ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
 
 char	*fgets(char *__str, int __size, FILE *__stream);
 char	*gets(char *__str);
@@ -299,15 +297,48 @@ int	ferror(FILE *__stream);
 #define FILENAME_MAX 1024
 #endif
 
-__extension__ typedef _fpos_t fpos_t;
-int fgetpos(FILE *stream, fpos_t *pos);
+/*
+ * Declare required C types
+ *
+ * size_t comes from stddef.h (included from cdefs.h)
+ */
+typedef _fpos_t fpos_t;
+
+#if __POSIX_VISIBLE
+/*
+ * Declare required additional POSIX types.
+ */
+
+# ifndef _OFF_T_DECLARED
+typedef	__off_t		off_t;		/* file offset */
+#  define _OFF_T_DECLARED
+# endif
+
+# ifndef _SSIZE_T_DECLARED
+typedef _ssize_t ssize_t;
+#  define _SSIZE_T_DECLARED
+# endif
+
+/* This needs to agree with <stdarg.h> */
+# ifdef __GNUC__
+#  ifndef _VA_LIST_DEFINED
+typedef __gnuc_va_list va_list;
+#   define _VA_LIST_DEFINED
+#  endif
+# else
+#  include <stdarg.h>
+# endif
+
+#endif
+
+int fgetpos(FILE * __restrict stream, fpos_t * __restrict pos);
 FILE *fopen(const char *path, const char *mode) __malloc_like_with_free(fclose, 1);
 FILE *freopen(const char *path, const char *mode, FILE *stream);
 FILE *fdopen(int, const char *) __malloc_like_with_free(fclose, 1);
 FILE *fmemopen(void *buf, size_t size, const char *mode) __malloc_like_with_free(fclose, 1);
 int fseek(FILE *stream, long offset, int whence);
 int fseeko(FILE *stream, __off_t offset, int whence);
-int fsetpos(FILE *stream, fpos_t *pos);
+int fsetpos(FILE *stream, const fpos_t *pos);
 long ftell(FILE *stream);
 __off_t ftello(FILE *stream);
 int fileno(FILE *);
@@ -321,8 +352,51 @@ void setlinebuf(FILE *stream);
 int setvbuf(FILE *stream, char *buf, int mode, size_t size);
 FILE *tmpfile(void);
 char *tmpnam (char *s);
-ssize_t getline(char **__restrict lineptr, size_t *__restrict n, FILE *__restrict stream);
-ssize_t getdelim(char **__restrict lineptr, size_t *__restrict  n, int delim, FILE *__restrict stream);
+_ssize_t getline(char **__restrict lineptr, size_t *__restrict n, FILE *__restrict stream);
+_ssize_t getdelim(char **__restrict lineptr, size_t *__restrict  n, int delim, FILE *__restrict stream);
+
+#if __BSD_VISIBLE
+FILE	*funopen (const void *cookie,
+		_ssize_t (*readfn)(void *cookie, void *buf,
+				size_t n),
+		_ssize_t (*writefn)(void *cookie, const void *buf,
+				 size_t n),
+		__off_t (*seekfn)(void *cookie, __off_t off, int whence),
+		int (*closefn)(void *cookie));
+# define	fropen(__cookie, __fn) funopen(__cookie, __fn, NULL, NULL, NULL)
+# define	fwopen(__cookie, __fn) funopen(__cookie, NULL, __fn, NULL, NULL)
+#endif /*__BSD_VISIBLE */
+
+#if __POSIX_VISIBLE >= 199309L
+int	getc_unlocked (FILE *);
+int	getchar_unlocked (void);
+void	flockfile (FILE *);
+int	ftrylockfile (FILE *);
+void	funlockfile (FILE *);
+int	putc_unlocked (int, FILE *);
+int	putchar_unlocked (int);
+#define getc_unlocked(f) fgetc(f)
+#define getchar_unlocked(f) fgetc(stdin)
+#define putc_unlocked(c, f) fputc(c, f)
+#define putchar_unlocked(c, f) fgetc(c, stdin)
+#endif
+
+#if __STDC_WANT_LIB_EXT1__ == 1
+#include <sys/_types.h>
+
+#ifndef _ERRNO_T_DEFINED
+typedef __errno_t errno_t;
+#define _ERRNO_T_DEFINED
+#endif
+
+#ifndef _RSIZE_T_DEFINED
+typedef __rsize_t rsize_t;
+#define _RSIZE_T_DEFINED
+#endif
+
+int sprintf_s(char *__restrict __s, rsize_t __bufsize,
+              const char *__restrict __format, ...);
+#endif
 
 /*
  * The format of tmpnam names is TXXXXXX, which works with mktemp
@@ -338,20 +412,18 @@ ssize_t getdelim(char **__restrict lineptr, size_t *__restrict  n, int delim, FI
  * We don't have any way of knowing any underlying POSIX limits,
  * so just use a reasonably small value here
  */
+#ifndef TMP_MAX
 #define TMP_MAX         32
-
-#ifdef __cplusplus
-}
 #endif
 
 /*@}*/
 
-static __inline uint32_t
+static __inline __uint32_t
 __printf_float(float f)
 {
 	union {
 		float		f;
-		uint32_t	u;
+		__uint32_t	u;
 	} u = { .f = f };
 	return u.u;
 }
@@ -423,6 +495,12 @@ __printf_float(float f)
 # define _HAS_IO_POS_ARGS
 # define _HAS_IO_C99_FORMATS
 # define _HAS_IO_DOUBLE
+# if defined(_MB_CAPABLE) || defined(_WANT_IO_WCHAR)
+#  define _HAS_IO_WCHAR
+# endif
+# ifdef _MB_CAPABLE
+#  define _HAS_IO_MBCHAR
+# endif
 # ifdef _WANT_IO_PERCENT_B
 #  define _HAS_IO_PERCENT_B
 # endif
@@ -431,6 +509,8 @@ __printf_float(float f)
 # endif
 #endif
 
+_END_STD_C
+
 #if __SSP_FORTIFY_LEVEL > 0
 #include <ssp/stdio.h>
 #endif
