@@ -45,6 +45,7 @@
 #include <wchar.h>
 #include <float.h>
 #include <math.h>
+#include <limits.h>
 #include <stdio-bufio.h>
 #include <sys/lock.h>
 
@@ -134,6 +135,70 @@ bool __matchcaseprefix(const char *input, const char *pattern);
                 .size = 0,                      \
 	}
 
+#define _FDEV_BUFIO_FD(bf) ((int)((intptr_t) (bf)->ptr))
+
+
+/*
+ * While there are notionally two different ways to invoke the
+ * callbacks (one with an int and the other with a pointer), they are
+ * functionally identical on many architectures. Check for that and
+ * skip the extra code.
+ */
+#if __SIZEOF_POINTER__ == __SIZEOF_INT__ || defined(__x86_64) || defined(__arm__) || defined(__riscv)
+#define BUFIO_ABI_MATCHES
+#endif
+
+/* Buffered I/O routines for tiny stdio */
+
+static inline ssize_t bufio_read(struct __file_bufio *bf, void *buf, size_t count)
+{
+#ifndef BUFIO_ABI_MATCHES
+    if (!(bf->bflags & __BFPTR))
+        return (bf->read_int)(_FDEV_BUFIO_FD(bf), buf, count);
+#endif
+    return (bf->read_ptr)((void *) bf->ptr, buf, count);
+}
+
+static inline ssize_t bufio_write(struct __file_bufio *bf, const void *buf, size_t count)
+{
+#ifndef BUFIO_ABI_MATCHES
+    if (!(bf->bflags & __BFPTR))
+        return (bf->write_int)(_FDEV_BUFIO_FD(bf), buf, count);
+#endif
+    return (bf->write_ptr)((void *) bf->ptr, buf, count);
+}
+
+static inline __off_t bufio_lseek(struct __file_bufio *bf, __off_t offset, int whence)
+{
+#ifndef BUFIO_ABI_MATCHES
+    if (!(bf->bflags & __BFPTR)) {
+        if (bf->lseek_int)
+            return (bf->lseek_int)(_FDEV_BUFIO_FD(bf), offset, whence);
+    } else
+#endif
+    {
+        if (bf->lseek_ptr)
+            return (bf->lseek_ptr)((void *) bf->ptr, offset, whence);
+    }
+    return _FDEV_ERR;
+}
+
+static inline int bufio_close(struct __file_bufio *bf)
+{
+    int ret = 0;
+#ifndef BUFIO_ABI_MATCHES
+    if (!(bf->bflags & __BFPTR)) {
+        if (bf->close_int)
+            ret = (bf->close_int)(_FDEV_BUFIO_FD(bf));
+    } else
+#endif
+    {
+        if (bf->close_ptr)
+            ret = (bf->close_ptr)((void *) bf->ptr);
+    }
+    return ret;
+}
+
 #ifdef POSIX_IO
 
 #define FDEV_SETUP_POSIX(fd, buf, size, rwflags, bflags)        \
@@ -160,10 +225,27 @@ __stdio_sflags (const char *mode);
 
 int	__d_vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(printf, 2, 0);
 int	__f_vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__i_vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__l_vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__m_vfprintf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+
 int	__d_sprintf(char *__s, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 2, 0);
 int	__f_sprintf(char *__s, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__i_sprintf(char *__s, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__l_sprintf(char *__s, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+int	__m_sprintf(char *__s, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 2, 0);
+
 int	__d_snprintf(char *__s, size_t __n, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 3, 0);
 int	__f_snprintf(char *__s, size_t __n, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 3, 0);
+int	__i_snprintf(char *__s, size_t __n, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 3, 0);
+int	__l_snprintf(char *__s, size_t __n, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 3, 0);
+int	__m_snprintf(char *__s, size_t __n, const char *__fmt, ...) __FORMAT_ATTRIBUTE__(printf, 3, 0);
+
+int	__d_vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	__f_vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	__i_vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	__l_vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
+int	__m_vfscanf(FILE *__stream, const char *__fmt, va_list __ap) __FORMAT_ATTRIBUTE__(scanf, 2, 0);
 
 #if __SIZEOF_DOUBLE__ == 8
 #define FLOAT64 double
@@ -580,6 +662,12 @@ __non_atomic_compare_exchange_ungetc(__ungetc_t *p, __ungetc_t d, __ungetc_t v)
 	return true;
 }
 
+static inline uint16_t
+__non_atomic_load_ungetc(const volatile __ungetc_t *p)
+{
+        return *p;
+}
+
 #ifdef ATOMIC_UNGETC
 
 #if __PICOLIBC_UNGETC_SIZE == 4 && defined (__GCC_HAVE_SYNC_COMPARE_AND_SWAP_4)
@@ -608,6 +696,12 @@ __atomic_exchange_ungetc(__ungetc_t *p, __ungetc_t v)
 	return atomic_exchange_explicit(pa, v, memory_order_relaxed);
 }
 
+static inline __ungetc_t
+__atomic_load_ungetc(const volatile __ungetc_t *p)
+{
+	_Atomic __ungetc_t *pa = (_Atomic __ungetc_t *) p;
+        return atomic_load(pa);
+}
 #else
 
 bool
@@ -616,6 +710,19 @@ __atomic_compare_exchange_ungetc(__ungetc_t *p, __ungetc_t d, __ungetc_t v);
 __ungetc_t
 __atomic_exchange_ungetc(__ungetc_t *p, __ungetc_t v);
 
+__ungetc_t
+__atomic_load_ungetc(const volatile __ungetc_t *p);
+
+__ungetc_t
+__picolibc_non_atomic_load_ungetc(const volatile __ungetc_t *p);
+
+__ungetc_t
+__picolibc_non_atomic_exchange_ungetc(__ungetc_t *p, __ungetc_t v);
+
+bool
+__picolibc_non_atomic_compare_exchange_ungetc(__ungetc_t *p,
+                                              __ungetc_t d, __ungetc_t v);
+
 #endif /* PICOLIBC_HAVE_SYNC_COMPARE_AND_SWAP */
 
 #else
@@ -624,6 +731,8 @@ __atomic_exchange_ungetc(__ungetc_t *p, __ungetc_t v);
 
 #define __atomic_exchange_ungetc(p,v) __non_atomic_exchange_ungetc(p,v)
 
+#define __atomic_load_ungetc(p) (*(p))
+
 #endif /* ATOMIC_UNGETC */
 
 /*
